//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentBoundingBox.cpp: implementation of the AgentBoundingBox class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MELUtilities.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentBoundingBox.h"
#include "AgentBoundingBoxDialog.h"
#include "AgentControl.h"
#include "AgentType.h"
#include "ZoneTypes.h"
#include <math.h>
#include <iostream>
#include <sstream>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentBoundingBox::AgentBoundingBox(int zoneType /*= 0*/)
:Agent(zoneType)
{
/*	_minX = BB_MIN_X;
	_minY = BB_MIN_Y;
	_minZ = BB_MIN_Z;
	_maxX = BB_MAX_X;
	_maxY = BB_MAX_Y;
	_maxZ = BB_MAX_Z;

	_XWeight = BB_XWEIGHT;
	_YWeight = BB_YWEIGHT;
	_ZWeight = BB_ZWEIGHT;
*/
	fullType.push_front(getLocalType());
	receiveSettings();

	loadAnalyses();
}

AgentBoundingBox::~AgentBoundingBox()
{

}
 
string    AgentBoundingBox::getLocalType()
{
	 return "AgentBoundingBox";
 
}

void	AgentBoundingBox::loadAnalyses() {

	boundingAnalysis = (AnalysisBoundingBox*)(AgentControl::getCurrentDesign()->getAnalysis("AnalysisBoundingBox"));

}

bool	AgentBoundingBox::step(int stepNum) {

	bool any;
	double testMinX;
	double testMinY;
	double testMinZ;
	double testMaxX;
	double testMaxY;
	double testMaxZ;

	int minX = _centerX - 50 - _sideX/2;
	int maxX = _centerX - 50 + _sideX/2;
	int minY = _centerY - 50 - _sideY/2;
	int maxY = _centerY - 50 + _sideY/2;
	int minZ = _centerZ - 50 - _sideZ/2;
	int maxZ = _centerZ - 50 + _sideZ/2;

	boundingAnalysis->getBounds(any, testMinX, testMinY, testMinZ, testMaxX, testMaxY, testMaxZ, _zoneType);
	if (any) {
		if (testMinX < minX) impression += (minX - testMinX) * _XWeight / 2.0;
		if (testMinY < minY) impression += (minY - testMinY) * _YWeight / 2.0;
		if (testMinZ < minZ) impression += (minZ - testMinZ) * _ZWeight / 2.0;
		if (testMaxX > maxX) impression += (testMaxX - maxX) * _XWeight / 2.0;
		if (testMaxY > maxY) impression += (testMaxY - maxY) * _YWeight / 2.0;
		if (testMaxZ > maxZ) impression += (testMaxZ - maxZ) * _ZWeight / 2.0;

		if (_XWeight + _YWeight + _ZWeight == 0.0) impression = 0;
		else impression /= (_XWeight + _YWeight + _ZWeight);

		_valid = true;
	
	}
	else {
		_valid = false;
		impression = 0;
	}

	return false;
}

Agent*	AgentBoundingBox::newOne(int zoneType /*= 0*/)
{
	return new AgentBoundingBox(zoneType);
}

void	AgentBoundingBox::copyProperties(Agent* original)
{
	AgentBoundingBox* realOriginal = (AgentBoundingBox*)original;

	_sideX = realOriginal->_sideX;
	_sideY = realOriginal->_sideY;
	_sideZ = realOriginal->_sideZ;
	_centerX = realOriginal->_sideX;
	_centerY = realOriginal->_centerY;
	_centerZ = realOriginal->_centerZ;

	_XWeight = realOriginal->_XWeight;
	_YWeight = realOriginal->_YWeight;
	_ZWeight = realOriginal->_ZWeight;
}

void	AgentBoundingBox::doInterface()
{
	AgentBoundingBoxDialog	modifyDialog;
	modifyDialog.setXside(_sideX);
	modifyDialog.setXcenter(_centerX);
	modifyDialog.setYside(_sideY);
	modifyDialog.setYcenter(_centerY);
	modifyDialog.setZside(_sideZ);
	modifyDialog.setZcenter(_centerZ);
	modifyDialog.setXWeight(_XWeight);
	modifyDialog.setYWeight(_YWeight);
	modifyDialog.setZWeight(_ZWeight);
	
	modifyDialog.setZoneName(ZoneTypes::zoneTypes->getZoneName(_zoneType));

	if (modifyDialog.DoModal() == IDOK) {
		_sideX = modifyDialog.getXside();
		_centerX = modifyDialog.getXcenter();
		_sideY = modifyDialog.getYside();
		_centerY = modifyDialog.getYcenter();
		_sideZ = modifyDialog.getZside();
		_centerZ = modifyDialog.getZcenter();
		_XWeight = modifyDialog.getXWeight();
		_YWeight = modifyDialog.getYWeight();
		_ZWeight = modifyDialog.getZWeight();

		if (modifyDialog.getCopyAcross())
			_myType->copyAgentSettingsAcrossAllAgents(this);

		modifyDialog.deleteBoundingBox();
	}
}

void		AgentBoundingBox::reportSettings()
{
	Settings::settings->setSetting(getLocalType(), "sideX", _sideX, _zoneType);
	Settings::settings->setSetting(getLocalType(), "sideY", _sideY, _zoneType);
	Settings::settings->setSetting(getLocalType(), "sideZ", _sideZ, _zoneType);
	Settings::settings->setSetting(getLocalType(), "centerX", _centerX, _zoneType);
	Settings::settings->setSetting(getLocalType(), "centerY", _centerY, _zoneType);
	Settings::settings->setSetting(getLocalType(), "centerZ", _centerZ, _zoneType);
	Settings::settings->setSetting(getLocalType(), "WeightX", _XWeight, _zoneType);
	Settings::settings->setSetting(getLocalType(), "WeightY", _YWeight, _zoneType);
	Settings::settings->setSetting(getLocalType(), "WeightZ", _ZWeight, _zoneType);
}

void		AgentBoundingBox::receiveSettings()
{
	int tempInt;
	double tempDouble;

	if (Settings::settings->getSetting(getLocalType(), "centerX", tempDouble, _zoneType)) {
		_centerX = tempDouble;
	}
	else _centerX = BB_CENTER;

	if (Settings::settings->getSetting(getLocalType(), "centerY", tempDouble, _zoneType)) {
		_centerY = tempDouble;
	}
	else _centerY = BB_CENTER;

	if (Settings::settings->getSetting(getLocalType(), "centerZ", tempDouble, _zoneType)) {
		_centerZ = tempDouble;
	}
	else _centerZ = BB_CENTER;

	if (Settings::settings->getSetting(getLocalType(), "sideX", tempDouble, _zoneType)) {
		_sideX = tempDouble;
	}
	else _sideX = BB_SIDE;

	if (Settings::settings->getSetting(getLocalType(), "sideY", tempDouble, _zoneType)) {
		_sideY = tempDouble;
	}
	else _sideY = BB_SIDE;

	if (Settings::settings->getSetting(getLocalType(), "sideZ", tempDouble, _zoneType)) {
		_sideZ = tempDouble;
	}
	else _sideZ = BB_SIDE;

	if (Settings::settings->getSetting(getLocalType(), "WeightX", tempInt, _zoneType)) {
		_XWeight = tempInt;
	}
	else _XWeight = BB_XWEIGHT;
	
	if (Settings::settings->getSetting(getLocalType(), "WeightY", tempInt, _zoneType)) {
		_YWeight = tempInt;
	}
	else _YWeight = BB_YWEIGHT;
	
	if (Settings::settings->getSetting(getLocalType(), "WeightZ", tempInt, _zoneType)) {
		_ZWeight = tempInt;
	}
	else _ZWeight = BB_ZWEIGHT;
}