//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentBoundingBoxDialog.cpp : implementation file
//

#include "stdafx.h"
#include "MFCPlugin.h"
#include "MELUtilities.h"
#include "MayaUtilities.h"
#include "AgentBoundingBoxDialog.h"
#include <iostream>
#include <sstream>
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// AgentBoundingBoxDialog dialog


AgentBoundingBoxDialog::AgentBoundingBoxDialog(CWnd* pParent /*=NULL*/)
	: CDialog(AgentBoundingBoxDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(AgentBoundingBoxDialog)
	m_copy_bounding = FALSE;
	m_xweight = 0;
	m_yweight = 0;
	m_zweight = 0;
	m_xcenter = 0;
	m_xside = 0;
	m_ycenter = 0;
	m_yside = 0;
	m_zcenter = 0;
	m_zside = 0;
	//}}AFX_DATA_INIT
}


void AgentBoundingBoxDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(AgentBoundingBoxDialog)
	DDX_Check(pDX, IDC_COPY_BOUNDING_PROPS_ACROSS_ZONETYPES, m_copy_bounding);
	DDX_Text(pDX, IDC_BB_XWEIGHT, m_xweight);
	DDV_MinMaxInt(pDX, m_xweight, 0, 100);
	DDX_Text(pDX, IDC_BB_YWEIGHT, m_yweight);
	DDV_MinMaxInt(pDX, m_yweight, 0, 100);
	DDX_Text(pDX, IDC_BB_ZWEIGHT, m_zweight);
	DDV_MinMaxInt(pDX, m_zweight, 0, 100);
	DDX_Slider(pDX, IDC_XCENTER, m_xcenter);
	DDX_Slider(pDX, IDC_XSIDE, m_xside);
	DDX_Slider(pDX, IDC_YCENTER, m_ycenter);
	DDX_Slider(pDX, IDC_YSIDE, m_yside);
	DDX_Slider(pDX, IDC_ZCENTER, m_zcenter);
	DDX_Slider(pDX, IDC_ZSIDE, m_zside);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(AgentBoundingBoxDialog, CDialog)
	//{{AFX_MSG_MAP(AgentBoundingBoxDialog)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_XCENTER, OnCustomdrawXcenter)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_XSIDE, OnCustomdrawXside)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_YCENTER, OnCustomdrawYcenter)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_YSIDE, OnCustomdrawYside)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_ZCENTER, OnCustomdrawZcenter)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_ZSIDE, OnCustomdrawZside)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// AgentBoundingBoxDialog message handlers

BOOL AgentBoundingBoxDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	((CStatic*)GetDlgItem(IDC_BB_FOR_ZONE_TYPE))->SetWindowText(("Bounding Box Settings for ZoneType: " + _zoneName).c_str());

	drawBoundingBox();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void AgentBoundingBoxDialog::OnCancel() 
{
	deleteBoundingBox();
	
	CDialog::OnCancel();
}


void AgentBoundingBoxDialog::drawBoundingBox()
{
	ostringstream Xside, Yside, Zside, Xpos, Ypos, Zpos;
	setLayerVisibility("defaultLayer", true);
	setCurrentLayer("defaultLayer");
	if(m_xside!=0)
		Xside << m_xside;
	else
		Xside << 0.001;
	if(m_yside!=0)
		Yside << m_yside;
	else
		Yside << 0.001;
	if(m_zside!=0)
		Zside << m_zside;
	else
		Zside << 0.001;

	eval("polyCube -ch off -n AgentBoundingBoxCube -w " + Xside.str() + " -h " + Yside.str() + " -d " + Zside.str());
	
	int tmpX, tmpY, tmpZ;
	tmpX = m_xcenter-50;//-m_xside/2;
	tmpY = m_ycenter-50;//-m_yside/2;
	tmpZ = m_zcenter-50;//-m_zside/2;
	Xpos << fabs(tmpX);
	Ypos << fabs(tmpY);
	Zpos << fabs(tmpZ);
	
	if(tmpX >= 0)
		eval("setAttr AgentBoundingBoxCube.translateX " + Xpos.str());
	else
		eval("setAttr AgentBoundingBoxCube.translateX -" + Xpos.str());
	if(tmpY>=0)
		eval("setAttr AgentBoundingBoxCube.translateY " + Ypos.str());
	else
		eval("setAttr AgentBoundingBoxCube.translateY -" + Ypos.str());
	if(tmpZ>=0)
		eval("setAttr AgentBoundingBoxCube.translateZ " + Zpos.str());
	else
		eval("setAttr AgentBoundingBoxCube.translateZ -" + Zpos.str());
	
	eval("refresh");
}

void AgentBoundingBoxDialog::deleteBoundingBox()
{
	eval("delete AgentBoundingBoxCube");
	setLayerVisibility("defaultLayer", false);
	eval("refresh");
}

void AgentBoundingBoxDialog::OnCustomdrawXcenter(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	deleteBoundingBox();
	drawBoundingBox();
	
	*pResult = 0;
}

void AgentBoundingBoxDialog::OnCustomdrawXside(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	deleteBoundingBox();
	drawBoundingBox();
	
	*pResult = 0;
}

void AgentBoundingBoxDialog::OnCustomdrawYcenter(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	deleteBoundingBox();
	drawBoundingBox();
	
	*pResult = 0;
}

void AgentBoundingBoxDialog::OnCustomdrawYside(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	deleteBoundingBox();
	drawBoundingBox();
	
	*pResult = 0;
}

void AgentBoundingBoxDialog::OnCustomdrawZcenter(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	deleteBoundingBox();
	drawBoundingBox();
	
	*pResult = 0;
}

void AgentBoundingBoxDialog::OnCustomdrawZside(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdateData();
	deleteBoundingBox();
	drawBoundingBox();
	
	*pResult = 0;
}
