// AgentDesignHeight.cpp: implementation of the AgentDesignHeight class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentDesignHeight.h"
#include "AgentControl.h"
#include "AgentDesignHeightDialog.h"
#include "AgentType.h"
#include <iostream>
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentDesignHeight::AgentDesignHeight(int zoneType /*= 0*/)
:Agent(zoneType)
{
	_targetHeight = INITIAL_TARGET_HEIGHT;

	loadAnalyses();
}

AgentDesignHeight::AgentDesignHeight(const AgentDesignHeight& original)
: Agent(original)
{
	_targetHeight = original._targetHeight;
}

AgentDesignHeight::~AgentDesignHeight()
{
}


void	AgentDesignHeight::loadAnalyses() {
	heightAnalysis = (AnalysisMaxHeight*)(AgentControl::getCurrentDesign()->getAnalysis("AnalysisMaxHeight"));
}

bool	AgentDesignHeight::step(int stepNum) {

	bool any;
	double height = heightAnalysis->getHeight(any, _zoneType);
	if (any) {
		impression = fabs(_targetHeight - height);

		_valid = true;
	}
	else {
		_valid = false;
		impression = 0;
	}
	return false;
}

Agent*	AgentDesignHeight::newOne(int zoneType /*= 0*/)
{
	 AgentDesignHeight* newAgent = new AgentDesignHeight(zoneType);
	 return newAgent;
}

void	AgentDesignHeight::copyProperties(Agent* original)
{
	AgentDesignHeight* realOriginal = (AgentDesignHeight*)original;

	_targetHeight = realOriginal->_targetHeight;
}

void	AgentDesignHeight::doInterface()
{
	AgentDesignHeightDialog	modifyDialog;
	modifyDialog.setTarget(_targetHeight);
	if (modifyDialog.DoModal() == IDOK) {
		_targetHeight = modifyDialog.getTarget();

		if (modifyDialog.getCopyAcross())
			_myType->copyAgentSettingsAcrossAllAgents(this);
	}
}