//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentProperties.cpp : implementation file
//

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AgentProperties.h"
#include "Constants.h"
#include "AgentControl.h"
#include "AgentType.h"
#include "GeneticEngine.h"
#include "ZoneTypes.h"
#include "PopulationProperties.h"
#include <iostream>
#include <sstream>

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// AgentProperties dialog

AgentProperties::AgentProperties()
	: CPropertyPage(AgentProperties::IDD)
{
	//{{AFX_DATA_INIT(AgentProperties)
	m_num_deployed = 0;
	m_weight = 0;
	//}}AFX_DATA_INIT
}


void AgentProperties::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(AgentProperties)
	DDX_Text(pDX, IDC_NUMBER_DEPLOYED, m_num_deployed);
	DDV_MinMaxInt(pDX, m_num_deployed, 0, 1000);
	DDX_Text(pDX, IDC_WEIGHT, m_weight);
	DDV_MinMaxInt(pDX, m_weight, -100, 100);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(AgentProperties, CPropertyPage)
	//{{AFX_MSG_MAP(AgentProperties)
	ON_CBN_SELENDOK(IDC_AGENT_ZONE_TYPE, OnSelendokAgentZoneType)
	ON_LBN_SELCHANGE(IDC_AGENT_TYPE_LIST, OnSelchangeAgentTypeList)
	ON_EN_KILLFOCUS(IDC_WEIGHT, OnKillfocusWeight)
	ON_EN_KILLFOCUS(IDC_NUMBER_DEPLOYED, OnKillfocusNumberDeployed)
	ON_BN_CLICKED(IDC_EDIT_AGENT, OnEditAgent)
	ON_BN_CLICKED(IDC_COPY_SETTINGS_ACROSS_ZONE_TYPES, OnCopySettingsAcrossZoneTypes)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// AgentProperties message handlers

BOOL AgentProperties::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	CListBox* typeList = (CListBox*)GetDlgItem(IDC_AGENT_TYPE_LIST);

	int tabStop[2] = {90, 120};
	typeList->SetTabStops( 2, tabStop );

	((CStatic*)GetDlgItem(IDC_IMPRESSION))->SetWindowText(_impression.c_str());

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void AgentProperties::OnSelendokAgentZoneType() 
{
	CComboBox* zoneList = (CComboBox*)GetDlgItem(IDC_AGENT_ZONE_TYPE);
 	currentZone = zoneList->GetCurSel() - 1;
	fillAgentTypeList(currentZone);

	if (currentZone < 0)
		(CButton*)GetDlgItem(IDC_COPY_SETTINGS_ACROSS_ZONE_TYPES)->EnableWindow(false);
	else
		(CButton*)GetDlgItem(IDC_COPY_SETTINGS_ACROSS_ZONE_TYPES)->EnableWindow(true);

	OnSelchangeAgentTypeList();
}

void AgentProperties::fillAgentTypeList(int zoneType)
{
	ostringstream entry;

	typeSettingList.clear();
	
	CListBox* typeList = (CListBox*)GetDlgItem(IDC_AGENT_TYPE_LIST);
	typeList->ResetContent();

	AgentControl::theAgentControl->getAgentTypes(zoneType, typeSettingList);

	int i = 0;
	for (list<AgentType*>::iterator j = typeSettingList.begin(); j != typeSettingList.end(); j++) {
		entry.str("");
		entry << (*j)->getName() << '\t' << (*j)->getDeployed(zoneType) << '\t' << (*j)->getWeight(zoneType);
		typeList->AddString(entry.str().c_str());
		typeList->SetItemData(i, (DWORD)(*j));

		i++;
	}
	
	if (typeList->GetCount() > 0)
		typeList->SetCurSel(0);
	else
		typeList->SetCurSel(-1);
}

void AgentProperties::OnSelchangeAgentTypeList() 
{
	CListBox* typeList = (CListBox*)GetDlgItem(IDC_AGENT_TYPE_LIST);
	bool valid = false;
	ostringstream impr;
	impr << "Current Impression: ";
	double imp = AgentControl::theAgentControl->getImpression(valid, currentZone, ((AgentType*)(typeList->GetItemData(typeList->GetCurSel()))));
	if(valid && imp!=-1)
		impr << imp;
	_impression = impr.str();
	((CStatic*)GetDlgItem(IDC_IMPRESSION))->SetWindowText(_impression.c_str());

	if	(typeList->GetCurSel() >= 0) {
		m_num_deployed = ((AgentType*)(typeList->GetItemData(typeList->GetCurSel())))->getDeployed(currentZone);
		m_weight = ((AgentType*)(typeList->GetItemData(typeList->GetCurSel())))->getWeight(currentZone);
		(CEdit*)GetDlgItem(IDC_NUMBER_DEPLOYED)->EnableWindow(true);
		(CEdit*)GetDlgItem(IDC_WEIGHT)->EnableWindow(true);
		if (((AgentType*)(typeList->GetItemData(typeList->GetCurSel())))->hasInterface())
			(CButton*)GetDlgItem(IDC_EDIT_AGENT)->EnableWindow(true);
		else
			(CButton*)GetDlgItem(IDC_EDIT_AGENT)->EnableWindow(false);
	}
	else {
		m_num_deployed = 1;
		m_weight = 0;
		(CEdit*)GetDlgItem(IDC_NUMBER_DEPLOYED)->EnableWindow(false);
		(CEdit*)GetDlgItem(IDC_WEIGHT)->EnableWindow(false);
		(CButton*)GetDlgItem(IDC_EDIT_AGENT)->EnableWindow(false);
		(CButton*)GetDlgItem(IDC_COPY_SETTINGS_ACROSS_ZONE_TYPES)->EnableWindow(false);
	}

	UpdateData(false);
}

void AgentProperties::OnKillfocusWeight() 
{
	int oldAgentType;

	if (UpdateData(true)) {
		CListBox* typeList = (CListBox*)GetDlgItem(IDC_AGENT_TYPE_LIST);
		oldAgentType = typeList->GetCurSel();
		((AgentType*)(typeList->GetItemData(typeList->GetCurSel())))->setWeight(m_weight, currentZone);
		fillAgentTypeList(currentZone);
		typeList->SetCurSel(oldAgentType);
	}
}

void AgentProperties::OnKillfocusNumberDeployed() 
{
	int oldAgentType;

	if (UpdateData(true)) {
		CListBox* typeList = (CListBox*)GetDlgItem(IDC_AGENT_TYPE_LIST);
		oldAgentType = typeList->GetCurSel();
		((AgentType*)(typeList->GetItemData(typeList->GetCurSel())))->setDeployed(m_num_deployed, currentZone);
		fillAgentTypeList(currentZone);
		typeList->SetCurSel(oldAgentType);
	}
}

void AgentProperties::setEngine(GeneticEngine* newEngine) {
	theEngine = newEngine;
}

void AgentProperties::OnEditAgent() 
{
		CListBox* typeList = (CListBox*)GetDlgItem(IDC_AGENT_TYPE_LIST);
//		theEngine->getPopulation()->clearFitnesses();
		((AgentType*)(typeList->GetItemData(typeList->GetCurSel())))->doInterface(currentZone);
}

void AgentProperties::OnCopySettingsAcrossZoneTypes() 
{
	CListBox* typeList = (CListBox*)GetDlgItem(IDC_AGENT_TYPE_LIST);
//	theEngine->getPopulation()->clearFitnesses();
	((AgentType*)(typeList->GetItemData(typeList->GetCurSel())))->copyWeightAndDeploymentSettings(currentZone);
}

BOOL AgentProperties::OnSetActive() 
{
	CComboBox* zoneList = (CComboBox*)GetDlgItem(IDC_AGENT_ZONE_TYPE);
	zoneList->ResetContent();
	zoneList->AddString("General");
	for (int i = 0; i < ZoneTypes::zoneTypes->getNumZoneTypes(); i++) {
		zoneList->AddString(ZoneTypes::zoneTypes->getZoneName(i).c_str());
	}
	zoneList->SetCurSel(1);
	currentZone = 0;

	fillAgentTypeList(0);
	OnSelendokAgentZoneType();

	return CPropertyPage::OnSetActive();
}
