//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AnalysisLocalizedBoundingBox.cpp: implementation of the AnalysisLocalizedBoundingBox class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <maya/MGlobal.h>
#include "MFCPlugin.h"
#include "AnalysisLocalizedBoundingBox.h"
#include "Design.h"
#include "AgentControl.h"
#include "Shape.h"
#include "MELUtilities.h"
#include <iostream>
#include <string>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AnalysisLocalizedBoundingBox::AnalysisLocalizedBoundingBox()
{
	name = "AnalysisLocalizedBoundingBox";
}

AnalysisLocalizedBoundingBox::~AnalysisLocalizedBoundingBox()
{

}
void AnalysisLocalizedBoundingBox::getBoundaryRelations(vector<int>& numberOfIntersections, vector<int>& numberOfSurfaces, int zoneType)
//void AnalysisLocalizedBoundingBox::getBoundaryRelations(vector<BoundaryRelation>& bounds, vector<int>& zoneOfOtherCurve, const vector<BindingType>& bindingTypes, int zoneType)
{
	/*
	Design*	theDesign = AgentControl::getCurrentDesign();
	Phenotype*	thePhenome = &(theDesign->getPhenome());
	Shape*		currentShape;
	int			otherZoneType;
	
	bounds.clear();
	zoneOfOtherCurve.clear();

	for (list<Shape*>::iterator i = thePhenome->getShapes().begin(); i != thePhenome->getShapes().end(); i++) {
		cout << "Found shape named: " << (*i)->getName() << endl;
		if(zoneType == (*i)->getPEType().getType())	{
			for (list<Shape*>::iterator j = thePhenome->getShapes().begin(); j != thePhenome->getShapes().end(); j++) {
				otherZoneType = (*j)->getPEType().getType();
				if ( otherZoneType != zoneType && bindingTypes[otherZoneType] != NONE ) {
					bounds.push_back(FindIntersection((*i), (*j)));
					zoneOfOtherCurve.push_back(otherZoneType);
				}
			}	
		}
	}*/
	//Alternative method
	Design*	theDesign = AgentControl::getCurrentDesign();
	Phenotype*	thePhenome = &(theDesign->getPhenome());
	Shape*		currentShape;
	
	vector<string>	surfaces, tempVector;
	int intersection;
	string layer = thePhenome->getLayerName();
	vector<Shape*> tempList;

	eval("string $surfaces[] = `editDisplayLayerMembers -q " + layer + "`");
	getMELvalue("$surfaces", tempVector);

	//Some preprocessing to speed up the big loop ahead
	for (list<Shape*>::iterator g = thePhenome->getShapes().begin(); g != thePhenome->getShapes().end(); g++) {
		if((*g)->getLoftName().find("loftedSurface")==0)
			tempList.push_back((*g));
	}
	for(int h=0; h < tempVector.size(); h++)	{
		if(tempVector[h].find("loftedSurface")==0)
			surfaces.push_back(tempVector[h]);
	}

	for(int i=0; i < surfaces.size(); i++)	{
		//for (list<Shape*>::iterator j = thePhenome->getShapes().begin(); j != thePhenome->getShapes().end(); j++) {
		for(int j=0; j < tempList.size(); j++)	{
			//if((*j)->getLoftName()==surfaces[i] && (*j)->getPEType().getType()==zoneType)	{
			if(tempList[j]->getLoftName()==surfaces[i] && tempList[j]->getPEType().getType()==zoneType)	{
				for(int k=0; k < surfaces.size(); k++)	{
					if(k!=i)	{
						//for (list<Shape*>::iterator l = thePhenome->getShapes().begin(); l != thePhenome->getShapes().end(); l++) {
						for(int l=0; l < tempList.size(); l++)	{
							//if((*l)->getLoftName()==surfaces[k])	{
							if(tempList[l]->getLoftName()==surfaces[k])	{
								cout << "Checking " + surfaces[i] + " and " + surfaces[k] << ", with ZoneTypes " << tempList[j]->getPEType().getType() << " and " << tempList[l]->getPEType().getType() << endl;
								eval("int $intersection = catch(`intersect -ch off " + surfaces[i] + " " + surfaces[k] +"`)");
								getMELvalue("$intersection", intersection);
								if(intersection==0)	{
									cout << "Surfaces intersected" << endl;
									numberOfIntersections[tempList[j]->getPEType().getType()] += 1;
								}
							}
						}
					}
				}
			}
		}
	}
}

BoundaryRelation AnalysisLocalizedBoundingBox::FindIntersection(Shape *Boundary, Shape *Other)
{
	string boundaryName = Boundary->getName();
	string otherName = Other->getName();
	/*
	//make a copy that we can work with
	eval("duplicate -n boundaryCurve " + boundaryName);
	eval("duplicate -n otherCurve " + otherName);
	
	eval("closeCurve -ch off -rpo on boundaryCurve");	//close curves
	eval("closeCurve -ch off -rpo on otherCurve");

	eval("planarSrf -ch off -n BoundarySurface boundaryCurve");	//make surfaces
	eval("planarSrf -ch off -n OtherSurface otherCurve");

	int success;
	eval("int $success =`catch(intersect -ch off -n intersectionCurve BoundarySurface OtherSurface)`");
	getMELvalue("$success", success);
	eval("delete BoundarySurface");
	eval("delete OtherSurface");
	eval("delete boundaryCurve");
	eval("delete otherCurve");*/
	if(Boundary->intersects(Other))	{
		cout << "Shapes intersecting" << endl;
		return PARTIALLY;
	}
	else	{
		cout << "Curves not intersecting" << endl;
		//Get some attributes for the boundary
		double boundaryMinX, boundaryMinY, boundaryMinZ, boundaryMaxX, boundaryMaxY, boundaryMaxZ;
		
		eval("float $boundaryMinX = `getAttr " + boundaryName + ".boundingBoxMinX`");
		getMELvalue("$boundaryMinX", boundaryMinX);
		eval("float $boundaryMinY = `getAttr " + boundaryName + ".boundingBoxMinY`");
		getMELvalue("$boundaryMinY", boundaryMinZ);
		eval("float $boundaryMinZ = `getAttr " + boundaryName + ".boundingBoxMinZ`");
		getMELvalue("$boundaryMinZ", boundaryMinY);

		eval("float $boundaryMaxX = `getAttr " + boundaryName + ".boundingBoxMaxX`");
		getMELvalue("$boundaryMaxX", boundaryMaxX);
		eval("float $boundaryMaxY = `getAttr " + boundaryName + ".boundingBoxMaxY`");
		getMELvalue("$boundaryMaxY", boundaryMaxZ);
		eval("float $boundaryMaxZ = `getAttr " + boundaryName + ".boundingBoxMaxZ`");
		getMELvalue("$boundaryMaxZ", boundaryMaxY);

		double otherMinX, otherMinY, otherMinZ, otherMaxX, otherMaxY, otherMaxZ;
		
		eval("float $otherMinX = `getAttr " + otherName + ".boundingBoxMinX`");
		getMELvalue("$otherMinX", otherMinX);
		eval("float $otherMinY = `getAttr " + otherName + ".boundingBoxMinY`");
		getMELvalue("$otherMinY", otherMinZ);
		eval("float $otherMinZ = `getAttr " + otherName + ".boundingBoxMinZ`");
		getMELvalue("$otherMinZ", otherMinY);

		eval("float $otherMaxX = `getAttr " + otherName + ".boundingBoxMaxX`");
		getMELvalue("$otherMaxX", otherMaxX);
		eval("float $otherMaxY = `getAttr " + otherName + ".boundingBoxMaxY`");
		getMELvalue("$otherMaxY", otherMaxZ);
		eval("float $otherMaxZ = `getAttr " + otherName + ".boundingBoxMaxZ`");
		getMELvalue("$otherMaxZ", otherMaxY);


		//Check if boundary is larger than the intersection
		if(boundaryMaxX>otherMaxX && boundaryMaxY>otherMaxY /*&& boundaryMaxZ>otherMaxZ*/
			&& boundaryMinX<otherMinX && boundaryMinY<otherMinY /*&& boundaryMinZ<otherMinZ*/)
			return INSIDE;
		else
			return OUTSIDE;
	}
}
