//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// GeneticUtility.h: interface for the GeneticUtility class.
//
// GeneticUtility contains utility items used throughout Agency's 
// genetic engine.  These items cannot (I think) be placed in GeneticEngine.h/.c
// because they lead to circular dependencies which make a big-bada-boom.
// Param, Insertion, and ChangeTypes are enumerations that make it easy to 
// specify the type of mutation desired.  safeRandom conveniently (and abstractly 
// in case we change the random number generator) generates integral random 
// numbers between 0 and some max (not including the max) 
//////////////////////////////////////////////////////////////////////

#if !defined(GENETIC_UTILITY_H__INCLUDED_)
#define GENETIC_UTILITY_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


//Specifies the parameter to be modified in a parameter mutation
enum ParamType {PETYPE = 0, ZWIDENING = 1, SHAPEMODE = 2, ZHEIGHT = 3, ZSTART = 4};
//typedef enum {PETYPE = 0, ZWIDENING = 1, SHAPEMODE = 2, ZHEIGHT = 3, ZSTART = 4} ParamType;

//Specifies whether a new command will be added in the middle or end of the command list in an insertion mutation
enum InsertionType {MIDDLE, END};
//typedef enum {MIDDLE, END} InsertionType;

//Specifies the type of change to be made to a command in a change mutation
enum ChangeType {RING, PARAM, TYPE, COMPLETE};
//typedef enum {RING, PARAM, TYPE, COMPLETE} ChangeType;

//Utility Functions
int safeRandom(int max);  // use the Mersenne Twister random number generator to create a random number between 0 and max - 1
int randomRange(int min, int max);

//old definitions
//typedef enum {PARAM, INSERTION, CHANGE} MutationType;
//typedef enum {AGENCY_PETYPE, AGENCY_ZWIDENING, SHAPEMODE, ZHEIGHT, ZSTART} ParamType;
//typedef enum {AGENCY_MIDDLE, AGENCY_END} InsertionType;
//typedef enum {AGENCY_RING, AGENCY_PARAM, AGENCY_TYPE, AGENCY_COMPLETE} ChangeType;

#endif