//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// MayaInfo.cpp: implementation of the MayaInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MayaInfo.h"


#include <maya/NewMVector.h>
#include <maya/MSelectionList.h>
#include <maya/MItSelectionList.h>
#include <maya/MFnDagNode.h>
#include <maya/MDagModifier.h>
#include <iostream>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MayaInfo::MayaInfo()
{
	theName = "";
	theNode = NULL;
	theTransform = NULL;
}

MayaInfo::MayaInfo(const string& mayaName) {
	theName = mayaName;
	theNode = NULL;
	theTransform = NULL;
}

string& MayaInfo::name() {
	return theName;
}

MDagPath* MayaInfo::node() {
	if (theNode == NULL) makeTransform();
	return theNode;
}

MFnTransform* MayaInfo::transform() {
	if (theTransform == NULL) makeTransform();
	return theTransform;
}

void MayaInfo::clearTransform() {
	if (theNode != NULL) delete theNode;
	theNode = NULL;
	if (theTransform != NULL) delete theTransform;
	theTransform = NULL;
}

MayaInfo::~MayaInfo()
{
	clearTransform();
}

bool MayaInfo::setName(const string& newName) {
	theName = newName;
	clearTransform();
	return true;
}

bool MayaInfo::makeTransform() {
	MSelectionList      list;
	MStatus				status;
	MObject				transformNode;

	// Set Maya pointers to this object

	status = MGlobal::clearSelectionList ();
	if (!status) {
		status.perror("MGlobal::clearSelectionList");
		return false;
	}
	
	status = MGlobal::selectByName( theName.c_str() );
	if (!status) {
		status.perror("MGlobal::selectByName");
		return false;
	}
	
	status = MGlobal::getActiveSelectionList( list );
	if (!status) {
		status.perror("MGlobal::getActiveSelectionList");
		return false;
	}
	MItSelectionList listIter( list );

	theNode = new MDagPath();
	listIter.getDagPath( *theNode );

	transformNode = theNode->transform(&status);
	// This node has no transform - i.e., it's the world node
	if (!status && status.statusCode () == MStatus::kInvalidParameter) {
		if (theNode != NULL) delete theNode;
		theNode = NULL;
		theTransform = NULL;
		return false;
	}

	theTransform = new MFnTransform(transformNode, &status);
	if (!status) {
		status.perror("MFnTransform constructor");
		if (theNode != NULL) delete theNode;
		theNode = NULL;
		if (theTransform != NULL) delete theTransform;
		theTransform = NULL;
		return false;
	}

	return true;
}