//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// MayaUtilities.h: interface for the MayaUtilities.cpp file.
//
//////////////////////////////////////////////////////////////////////

#if !defined(MAYA_UTILITIES_H__INCLUDED_)
#define MAYA_UTILITIES_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include <vector>
#include <list>
#include "Shape.h"
#include "MayaInfo.h"
#include "EmergentShape.h"
using namespace std;

bool intersect(string curve1, string curve2);
bool planarIntersect(Shape* curve1, Shape* curve2);
void getIntersections(string curve1, string curve2, vector<double>& p1, vector<double>& p2);
void sliceCurve(string curve, vector<double>& cuts, vector<string>& newCurves);
void sewCurves(int method, const string& A, const string& B, const string& C, const string& D, vector<string>& newCurves);
void sewCurves(int method, const string& A, const string& B, const string& C, const string& D, vector<string>& newACurves, vector<string>& newBCurves);
void shuffleIntersections(int method, const string& curve1, const string& curve2, vector<string>& newACurves, vector<string>& newBCurves);
void shuffleIntersections(int method, Shape* curve1, Shape* curve2, vector<string>& newACurves, vector<string>& newBCurves);
void shuffleIntersections(int method, Shape* curve1, Shape* curve2, list<EmergentShape*>& newCurves);
bool duplicateCurve(MayaInfo* curve, string& newName);   // Makes a copy of the original Maya object to operate on safely
bool ZSection(Shape* A, Shape* B, Shape*& newA, Shape*& newB, ShapePair& intersectors, bool startFromZero);
void cutCurve(string curve, double start, double stop);

// Layer Utils
void deleteAllOnLayer(const string& name);
void selectAllOnLayer(const string& name);
bool layerExists(const string& name);
void newLayer(string& name);			// NOTE: Name will be changed if layer by "name" already exists
void setLayerVisibility(const string& name, bool vis);
void getCurrentLayer(string& name);
void setCurrentLayer(const string& name);

// Shader Utils
void initializeShaderGroups();


#endif // !defined(MAYA_UTILITIES_H__INCLUDED_)