//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// PrimaryShape.h: interface for the PrimaryShape class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PRIMARYSHAPE_H__B350BB9E_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
#define AFX_PRIMARYSHAPE_H__B350BB9E_E931_11D3_9F59_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "Shape.h"
#include "Command.h"
#include "PEType.h"
#include "GeneticUtility.h"
#include "Design.h"
#include "MayaInfo.h"

#include <list>
#include <string>

using namespace std;

class PrimaryShape : public Shape
{
public:
	PrimaryShape(MayaInfo* original = NULL, const string& originalLayer = "defaultLayer");
	PrimaryShape(const PrimaryShape& oldShape);
	virtual ~PrimaryShape();

// Genetic Operations
	void mutateParams(ParamType typeOfParam);
	void mutateInsertion(InsertionType typeOfInsertion);
	void mutateDeletion();
	void mutateChange(ChangeType typeOfChange);
	friend void PrimaryShape::crossover(int numberOfCrossingPoints, const Design &crossoverTemplate);
	void crossover(int numberOfCrossingPoints, Design* crossoverTemplate);

//  Interpretation tools	
	bool runNextCommand();
	void runCommand(Command* theCommand);
	virtual bool initializeInterpret();

// Accessors

	const list<Command*>& getOperations() const {return operations;};
	virtual bool isPrimary() {return true;};	// Is a primary shape
	virtual Shape*	fullDuplicate();

	PEType&	getOriginalPEType() {return originalAssignment;};
	double	getOriginalZWidening() {return originalZWidening;};
	int		getOriginalShapeMode() {return originalShapeMode;};
	double	getOriginalZHeight() {return originalZHeight;};
	double	getOriginalZStart() {return originalZStart;};

// Mutators

	void setOperations(const list<Command*>& newOps) {operations = newOps;};
	void clearOperations();
	void appendOperation(Command* newCommand);
	void removeOperation(Command* oldCommand);
	void insertOperationBefore(Command* insertionPoint, Command* newCommand);

	void	setOriginalPEType(int newType) {originalAssignment.setType(newType);};
	void	setOriginalZWidening(double newWidening) {originalZWidening = newWidening;};
	void	setOriginalShapeMode(int newMode) {originalShapeMode = newMode;};
	void	setOriginalZHeight(double newHeight) {originalZHeight = newHeight;};
	void	setOriginalZStart(double newStart) {originalZStart = newStart;};

	virtual void reduceNumZoneTypes(int newNumZoneTypes);

// Utility
	const PrimaryShape& operator=(const PrimaryShape& oldShape);


  // Transformations
	bool scale(double x, double y, double z);
	bool translate(double x, double y, double z);
	bool rotate(double degrees);
	bool opening(double start, double stop);

  // Object identification by name
//	bool setMayaName(const string& newName);
//	bool makeMayaTranform();  // Attach shape by name to Maya object
	void setOriginalLayer(string layerName)	{_originalLayer = layerName;};
	string getOriginalLayer()	{return _originalLayer;};

private:
//Other
	
	Command* createRandomCommand(); // Create a new random command based upon parameters in Constants.h
	//The functions below should be templatized and used regularly
	std::list<Command*>::iterator addIterator(std::list<Command*>::iterator startingPoint, int addend);
	std::list<Command*>::const_iterator addIterator(std::list<Command*>::const_iterator startingPoint, int addend); 


	list<Command*> operations; // The commands that the shape undergoes

	PEType originalAssignment;	// These are the shape parameters from the original design
	double originalZWidening;
	int originalShapeMode;
	double originalZHeight, originalZStart;

	string _originalLayer;

	// Maya representation variables
	MayaInfo*	originalCurve;  // The Maya info for the curve from the original design from which this shape is derived
	
	std::list<Command*>::iterator			nextCommand; // The next command to be run
};

#endif // !defined(AFX_PRIMARYSHAPE_H__B350BB9E_E931_11D3_9F59_0000E87A7A07__INCLUDED_)
