//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// SettingsProps.cpp : implementation file
//

#include "stdafx.h"
#include "GeneticEngine.h"
#include "MFCPlugin.h"
#include "SettingsProps.h"
#include "Settings.h"
#include <iostream>
#include <fstream>
#include "Random.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const int RAND_SEED = 42;

/////////////////////////////////////////////////////////////////////////////
// SettingsProps property page

IMPLEMENT_DYNCREATE(SettingsProps, CPropertyPage)

SettingsProps::SettingsProps() : CPropertyPage(SettingsProps::IDD)
{
	//{{AFX_DATA_INIT(SettingsProps)
	m_bRandSeed = FALSE;
	//}}AFX_DATA_INIT
	receiveSettings();
}

SettingsProps::~SettingsProps()
{
}

void SettingsProps::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SettingsProps)
	DDX_Check(pDX, IDC_RAND_SEED, m_bRandSeed);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(SettingsProps, CPropertyPage)
	//{{AFX_MSG_MAP(SettingsProps)
	ON_BN_CLICKED(IDC_LOAD_DEFAULT_SETTINGS, OnLoadDefaultSettings)
	ON_BN_CLICKED(IDC_LOAD_SETTINGS_FROM_FILE, OnLoadSettingsFromFile)
	ON_BN_CLICKED(IDC_SAVE_CURRENT_SETTINGS_AS, OnSaveCurrentSettingsAs)
	ON_BN_CLICKED(IDC_SAVE_SETTINGS_AS_DEFAULT, OnSaveSettingsAsDefault)
	ON_BN_CLICKED(IDC_WIPE_DEFAULTS, OnWipeDefaults)
	ON_BN_CLICKED(IDC_RAND_SEED, OnRandSeed)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// SettingsProps message handlers

void SettingsProps::OnLoadDefaultSettings() 
{
	Settings::setDefaultFile();
	Settings::readSettings();
	((CStatic*)GetDlgItem(IDC_CURRENT_SETTINGS_FILE))->SetWindowText(Settings::getSettingsFile().c_str());
}

void SettingsProps::OnLoadSettingsFromFile() 
{
	CFileDialog fileDialog(true, "agc", Settings::getSettingsFile().c_str());

	char filterBuf[] = "AgencyGP Settings Files (*.agc)\0*.agc\0\0";
	char titleBuf[] = "Open AgencyGP Settings File";

	fileDialog.m_ofn.lpstrFilter = filterBuf;
	fileDialog.m_ofn.lpstrTitle = titleBuf;
	fileDialog.m_ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

	if (fileDialog.DoModal() == IDOK) {
		Settings::setSettingsFile(fileDialog.GetPathName().GetBuffer(0));
		Settings::readSettings();
		((CStatic*)GetDlgItem(IDC_CURRENT_SETTINGS_FILE))->SetWindowText(Settings::getSettingsFile().c_str());
	}
	UpdateData(false);
}

void SettingsProps::OnSaveCurrentSettingsAs() 
{
	CFileDialog fileDialog(false, "agc", Settings::getSettingsFile().c_str());

	char filterBuf[] = "AgencyGP Settings Files (*.agc)\0*.agc\0\0";
	char titleBuf[] = "Save AgencyGP Settings File";

	fileDialog.m_ofn.lpstrFilter = filterBuf;
	fileDialog.m_ofn.lpstrTitle = titleBuf;
	fileDialog.m_ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY;

	if (fileDialog.DoModal() == IDOK) {
		Settings::setSettingsFile(fileDialog.GetPathName().GetBuffer(0));
		Settings::writeSettings();
		((CStatic*)GetDlgItem(IDC_CURRENT_SETTINGS_FILE))->SetWindowText(Settings::getSettingsFile().c_str());
	}
	
}

void SettingsProps::OnSaveSettingsAsDefault()
{
	Settings::setDefaultFile();
	Settings::writeSettings();
	((CStatic*)GetDlgItem(IDC_CURRENT_SETTINGS_FILE))->SetWindowText(Settings::getSettingsFile().c_str());
}

void SettingsProps::OnWipeDefaults() 
{
	Settings::setDefaultFile();

	ofstream newFile;
	newFile.open(Settings::getSettingsFile().c_str(), ios::out);
	newFile.close();
	Settings::readSettings();
	Settings::writeSettings();
	((CStatic*)GetDlgItem(IDC_CURRENT_SETTINGS_FILE))->SetWindowText(Settings::getSettingsFile().c_str());
}

BOOL SettingsProps::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	((CStatic*)GetDlgItem(IDC_CURRENT_SETTINGS_FILE))->SetWindowText(Settings::getSettingsFile().c_str());

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void SettingsProps::OnRandSeed() 
{
	time_t seed;
	UpdateData(true);
	if(m_bRandSeed)	seed = RAND_SEED;
	else	time( &seed );
	Random::sgenrand(seed);
	GeneticEngine::geneticEngine->SeedEngine();
}

void SettingsProps::reportSettings()
{
	Settings::settings->setSetting("SettingsProps", "RandomSeed", (int)m_bRandSeed);
}

void SettingsProps::receiveSettings()
{
	int tempInt;
	if (Settings::settings->getSetting("SettingsProps", "RandomSeed", tempInt)) {
		if(tempInt>=1)
			m_bRandSeed = true;
		else
			m_bRandSeed = false;
	}
	else m_bRandSeed = false;
}