//
// Copyright (C) 2001 Simon Greenwold. You are allowed to use or modify the code, but this note must be kept intact.
// 
// ZoneTypes.h: interface for the ZoneTypes class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ZONETYPES_H__E56FF881_67E5_11D4_9FB6_0000E87A7A07__INCLUDED_)
#define AFX_ZONETYPES_H__E56FF881_67E5_11D4_9FB6_0000E87A7A07__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Settable.h"
#include "GeneticEngine.h"
#include "AgentControl.h"
#include <list>
#include <vector>
#include <string>

using namespace std;

const int MUTATIONRATE = 100;

class AgentControl;

class ZoneTypes : public Settable  
{
public:
	ZoneTypes();
	virtual ~ZoneTypes();
	static ZoneTypes* zoneTypes;

	void addShader(const string& shader);
	void setNumZoneTypes(int newNumZoneTypes);

	void setZoneName(const string& newName, int i) {_zoneNames[i] = newName;};
	void setZoneShader(const string& newShader, int i) {_zoneShaders[i] = newShader;};
	void deleteZoneType(int typeNum);
	void addZoneType(const string& zoneName, const string& zoneShader, int typeNum);

	list<string>& getShaderGroups() {return _shaderGroups;};
	vector<string>& getZoneNames() {return _zoneNames;};
	vector<string>& getZoneShaders() {return _zoneShaders;};
	string getZoneName(int i) {if (i < 0) return "General"; return _zoneNames[i];};
	string getZoneShader(int i) {return _zoneShaders[i];};
	int    getNumZoneTypes() {return _numZoneTypes;};

	bool validShader(const string& testShader);

	void initializeZoneTypes();
	void setEngine(GeneticEngine* geneticEngine);
	void setAgentControl(AgentControl* agentControl);

	bool getMutableZoneType(int i)	{return _mutateInOut[i];};
	void setMutableZoneType(int i, bool mutate)	{_mutateInOut[i]=mutate;};

	int getMutationRate(int i)	{return _mutationRate[i];};
	void setMutationRate(int i, int mutationrate)	{_mutationRate[i] = mutationrate;};

	void checkLayerNames();
	int newZoneType(int i);
	
	
//	static bool isInitialized()	{return _initialized;};
	// Settables inheritance
protected:
//	static bool _initialized;

	virtual void reportSettings();
	virtual void receiveSettings();

private:

	list<string> _shaderGroups;
	int _numZoneTypes;
	vector<string> _zoneNames;
	vector<string> _zoneShaders;
	GeneticEngine* _geneticEngine;
	AgentControl*  _agentControl;

	vector<bool>	_mutateInOut;
	vector<int>		_mutationRate;
};

#endif // !defined(AFX_ZONETYPES_H__E56FF881_67E5_11D4_9FB6_0000E87A7A07__INCLUDED_)
